#!/bin/bash

# Define the file path
IFCFG_FILE="/etc/sysconfig/network-scripts/ifcfg-ens160"

# Check if the file already exists (for RHEL8 and older appliances)
if sudo test -f "$IFCFG_FILE"; then
    echo "The network configuration file already exists: $IFCFG_FILE"
else
    # Generate a new UUID
    NEW_UUID=$(uuidgen)
    # Create and write configurations to the file
    # Note: I have copied the content of this file from a RHEL8 system
    sudo tee "$IFCFG_FILE" > /dev/null <<EOF
TYPE=Ethernet
PROXY_METHOD=none
BROWSER_ONLY=no
BOOTPROTO=dhcp
DEFROUTE=yes
IPV4_FAILURE_FATAL=no
IPV6INIT=yes
IPV6_AUTOCONF=yes
IPV6_DEFROUTE=yes
IPV6_FAILURE_FATAL=no
IPV6_ADDR_GEN_MODE=default
NAME=ens160
UUID=$NEW_UUID
DEVICE=ens160
ONBOOT=yes
EOF

    # Set the correct ownership and permissions
    sudo chown root:root "$IFCFG_FILE"
    sudo chmod 644 "$IFCFG_FILE"

    # Restart the NetworkManager to apply changes
    sudo systemctl restart NetworkManager

    # Bring up the network interfaces
    sudo nmcli connection reload
    sudo nmcli connection up ens160

    echo "Network configuration for ens160 has been created with UUID: $NEW_UUID"
fi