#!/bin/bash



# Function to handle successful exit
clean_exit() {
  echo "Script completed successfully, Exiting!"
  exit 0
}


# Main Leap Upgrade function
leap_upgrade() {
  echo "Starting leap upgrade from RHEL 8 to 9 using ISO: $ISO_PATH"
  run_pre_upgrade_checks
  perform_upgrade
}

# Function to run pre-upgrade checks
run_pre_upgrade_checks() {
  echo "Running pre-upgrade checks..."
  # Pre Section
  echo "Preparing the system for Leapp preupgrade. Ensuring required configurations are applied."
  echo "Updating crypto policies to LEGACY..."
  update-crypto-policies --set LEGACY

  echo "Updating Firewalld configuration to disable AllowZoneDrifting..."
  sed -i 's/^AllowZoneDrifting=.*/AllowZoneDrifting=no/' /etc/firewalld/firewalld.conf
  echo "Answering required Leapp questions..."
  leapp answer --section check_vdo.confirm=True

  echo "Running leapp preupgrade..."
  echo "Using ISO_PATH: $ISO_PATH"
  leapp preupgrade --iso "$ISO_PATH" --no-rhsm --target 9.5 --verbose


  # Check for errors or inhibitors
  if grep -qiE "(inhibitor|error)" /var/log/leapp/leapp-report.txt; then
      echo "Errors or inhibitors detected during preupgrade. Stopping the script."
      exit 1
  fi


}

# Function to perform the upgrade
perform_upgrade() {
  echo "Performing the upgrade..."
  echo "Running leapp upgrade..."

  echo "Using ISO_PATH: $ISO_PATH"

  leapp upgrade --iso "$ISO_PATH" --no-rhsm --target 9.5 --verbose

  # Check for errors or inhibitors again
  if grep -qiE "(inhibitor|error)" /var/log/leapp/leapp-report.txt; then
      echo "Errors or inhibitors detected during upgrade. Stopping the script."
      exit 1
  fi
}

if [[ -z "$1" ]]; then
  echo "Error: Base directory path not provided."
  exit 1
fi

# Normalize the incoming path
BASE_PATH="${1%/}"  # Remove trailing slash if it exists

# Check if an .iso file exists in /data
COMPLETE_ISO_FILE=$(sudo find /data -maxdepth 1 -type f -name "*.iso" | head -n 1)

if [[ -n "$COMPLETE_ISO_FILE" ]]; then
  ISO_PATH="$COMPLETE_ISO_FILE"
else
  ISO_PATH="${BASE_PATH}/CUSTOM-RHEL9.5-x86_64.iso"
fi

# Print the chosen ISO path before performing the upgrade
echo "Final ISO_PATH: $ISO_PATH"

# Run the main function
leap_upgrade

clean_exit