#!/usr/bin/python3

##########################################################################
# Script runs product udpates
#####
# Syntax: update-manager.py --mode <checkupdate|update>
# Example: update-manager.py --mode update
#####
##########################################################################

from subprocess import call, check_output, check_call, Popen, PIPE
import string
import random
import os
import shutil
import logging
import sys
import re
import glob
from optparse import OptionParser, SUPPRESS_HELP
import urllib.request, urllib.parse, urllib.error
import urllib.request, urllib.error, urllib.parse
import json
import zipfile

CONST = None
FORWARD_SLASH = "/"
YUM = '/usr/bin/yum'
DNF = '/usr/bin/dnf'
UPDALT = '/usr/sbin/update-alternatives'
GREP = '/usr/bin/grep'
AWK = '/usr/bin/awk'
PING = '/bin/ping'
OSUPDATE_REPO_CONFIG = '/etc/yum.repos.d/osupdate.repo'
SPP_REPO_CONFIG = '/etc/yum.repos.d/spp.repo'

skip_if_not_installed = [
    "autoconf",
    "automake",
    "libtool",
    "ruby-devel",
    "openssl-devel",
    "zlib-devel",
    "libffi-devel",
]


class LessThenFilter(logging.Filter):
    def __init__(self, level):
        self._level = level
        logging.Filter.__init__(self)

    def filter(self, rec):
        return rec.levelno < self._level

def decode_output(stdout, stderr):
    if stdout is not None:
        stdout = stdout.decode('UTF-8')

    if stderr is not None:
        stderr = stderr.decode('UTF-8')

    return stdout, stderr

def setup():

    global logger

    logger = logging.getLogger('update')
    logger.setLevel(logging.DEBUG)

    # create a console handler
    cHandler = logging.StreamHandler(sys.stdout)
    cHandler.setLevel(logging.INFO)
    cHandler.setFormatter(logging.Formatter('%(message)s'))
    cHandler.addFilter(LessThenFilter(logging.WARNING))
    logger.addHandler(cHandler)

    # create a console handler for stderr
    ceHandler = logging.StreamHandler()
    ceHandler.setLevel(logging.ERROR)
    ceHandler.setFormatter(logging.Formatter('%(message)s'))
    # ceHandler.addFilter(LessThenFilter(logging.WARNING))
    logger.addHandler(ceHandler)

    # create a file handler
    fHandler = logging.FileHandler(options.log)
    fHandler.setLevel(logging.DEBUG)
    fHandler.setFormatter(
        logging.Formatter(
            '%(asctime)s - %(levelname)s - %(message)s',
            "%Y-%m-%d %H:%M:%S"))
    logger.addHandler(fHandler)

# Retry settings
MAX_RETRIES = 3  # Maximum number of retries
RETRY_DELAY = 5  # Delay between retries in seconds

def failed_exit():
    # A simple function to handle failures and exit
    logger.error('Download failed after multiple attempts.')
    exit(1)

def download_file(file_url, download_path='/tmp/downloaded_file.rpm'):
    logger.info('Downloading file from: %s', file_url)

    # Check if file already exists
    if not os.path.exists(download_path):
        retries = 0
        
        while retries < MAX_RETRIES:
            download_command = 'wget --progress=bar:force -O {0} {1}'.format(download_path, file_url)
            logger.info('Executing: %s', download_command)
            
            # Run the download command
            p = Popen(download_command, shell=True, stdout=PIPE, stderr=PIPE)
            stdout, stderr = p.communicate()
            
            # Check if the download was successful
            if p.returncode == 0:
                logger.info('File downloaded successfully to: %s', download_path)
                break  # Exit the loop if successful
            else:
                # Log error and retry if there's a failure  
                logger.error('Failed to download file. STDERR: %s', stderr.strip())
                retries += 1
                if retries < MAX_RETRIES:
                    logger.info('Retrying... (%d/%d)', retries, MAX_RETRIES)
                    time.sleep(RETRY_DELAY)  # Wait before retrying
                else:
                    failed_exit()  # Exit after max retries

    else:
        logger.info('File already exists at: %s', download_path)
def remove_package(package_name):
    yumrmcmd = YUM + ' remove -y ' + package_name
    logger.info('Running command: ' + yumrmcmd)
    p = Popen(yumrmcmd, stdout=PIPE, bufsize=1, shell=True, stderr=PIPE)
    stdout, stderr = p.communicate()
    stdout, stderr = decode_output(stdout, stderr)

def main():

    if options.yumclean:
        yumcleanall()

    if options.group is None:
        group = repo.get_group_name()
    else:
        group = options.group

    if options.mode == 'checkupdate':
        check_update(group)

    if options.mode == 'checkupgrade':
        check_upgrade()

    if options.mode == 'updateos':
        if options.upgrade == 'rhel9':
            return_code = 0
            if os.path.exists('/etc/yum.repos.d/local.repo'):
                logger.info("Local repo exists, triggering RHEL 9 upgrade through manual upgrade...")

                #Copying the ISO to a Persistently Mounted Parition - /tmp
                copy_service_cmd = 'cp {} {}'.format("/mnt/cslocalupdate/OS/Packages/CUSTOM-RHEL9.5-x86_64.iso", '/tmp/')
                os.popen(copy_service_cmd)

                return_code = rhel9Upgrade('/tmp/')
            elif os.path.exists(OSUPDATE_REPO_CONFIG) and is_ping(get_repo_server(OSUPDATE_REPO_CONFIG)):
                command = GREP + " 'baseurl' " + OSUPDATE_REPO_CONFIG
                logger.info( command )
                logger.info("OS Update repo is reachable, triggering path download through automatic upgrade...")
                file_url = "https://public.dhe.ibm.com/storage/spplus/10.1.17/OS/Packages/CUSTOM-RHEL9.5-x86_64.iso"
                download_path = "/tmp/CUSTOM-RHEL9.5-x86_64.iso"
                download_file(file_url, download_path)
                return_code = rhel9Upgrade('/tmp/')
            else:
                logger.error("Neither local.repo nor OSUPDATE_REPO_CONFIG is available. Exiting...")
                return_code = 1

            if return_code != 0:
                logger.error("Exiting the Upgrade Process: RHEL 9 Upgrade Unsuccessful")
                failed_exit()
        elif options.upgrade == 'winrm':
            spec_folder_cleanup_command = "sudo rm -rf /usr/local/share/gems/specifications/*"
            logger.info("Cleaning up /specifications/ folder")
            spec_folder_cleanup_output = os.popen(spec_folder_cleanup_command).read()
            logger.info(spec_folder_cleanup_output)
            
            # Uninstall old versions
            ffi_uninstall_command = "sudo gem uninstall ffi --force"
            logger.info("Uninstalling ffi gem")
            ffi_output = os.popen(ffi_uninstall_command).read()
            logger.info(ffi_output)
            
            gyoku_uninstall_command = "sudo gem uninstall gyoku --force"
            logger.info("Uninstalling gyoku gem")
            gyoku_output = os.popen(gyoku_uninstall_command).read()
            logger.info(gyoku_output)

            winrm_uninstall_command = "sudo gem uninstall winrm --force"
            logger.info("Uninstalling winrm gem")
            winrm_output = os.popen(winrm_uninstall_command).read()
            logger.info(winrm_output)
            
            winrm_fs_uninstall_command = "sudo gem uninstall winrm-fs --force"
            logger.info("Uninstalling winrm-fs gem")
            winrm_fs_output = os.popen(winrm_fs_uninstall_command).read()
            logger.info(winrm_fs_output)
            
            # Install new versions
            ffi_install_command = "sudo gem install ffi -v 1.15.5 --no-document --source https://rubygems.org"
            logger.info("Installing ffi gem version 1.15.5")
            ffi_install_output = os.popen(ffi_install_command).read()
            logger.info(ffi_install_output)
            
            winrm_install_command = "sudo gem install winrm -v 2.3.9 --no-document --source https://rubygems.org"
            logger.info("Installing winrm gem version 2.3.9")
            winrm_install_output = os.popen(winrm_install_command).read()
            logger.info(winrm_install_output)
            
            winrm_fs_install_command = "sudo gem install winrm-fs -v 1.1.0 --no-document --source https://rubygems.org"
            logger.info("Installing winrm-fs gem version 1.1.0")
            winrm_fs_output = os.popen(winrm_fs_install_command).read()
            logger.info(winrm_fs_output)
            
            gyoku_install_command = "sudo gem install gyoku -v 1.4.0 --no-document --source https://rubygems.org"
            logger.info("Installing gyoku gem version 1.4.0")
            gyoku_install_output = os.popen(gyoku_install_command).read()
            logger.info(gyoku_install_output)
            
            # Run gem pristine
            pristine_command = "sudo gem pristine --all"
            logger.info("Running gem pristine --all to restore gems to pristine condition")
            pristine_output = os.popen(pristine_command).read()
            logger.info(pristine_output)
            
            logger.info("Gem installation and cleanup steps completed.")
        else:
            if options.package is not None:
                update_or_install_package(options.package, False, options.skip_gpg_check)
            else:
                update_or_install_package('')

    if options.mode == 'install':
        if options.package is not None:
            install_package(options.package, True, options.install_type)

    if options.mode == 'update':
        # Installation VADP requires the latest VDDK, for this the VADP and VDDK needs to be removed first. 
        # After the above, the latest version can be installed.
        if options.package == 'vmware-vddk':
            remove_package(options.package)
            remove_package("vadp")
            update_or_install_package(options.package, True)
        elif options.package is not None:
            if options.package in skip_if_not_installed:
                if check_installed_package(options.package):
                    update_or_install_package(options.package, True)
                else:
                    logger.info(options.package + ' was not found on the system; skipping its upgrade.')
            else:
                update_or_install_package(options.package, True)

        elif options.group is None:

            all_groups = group
            if is_update_available('endeavour-platform'):
                all_groups += ' endeavour-platform'
            # special handle for vsnap  , the group name is 'vsnap'
            if is_update_available('vsnap'):

                update_or_install_package("vsnap-dist", True)

                run_vsnap_dist_installer()

            # update_or_install_package('spp-guestappvm', True)
            # update_or_install_package('spp-nmt', True)

            update_group(all_groups, True)

        else:
           # if  group == 'spp':
           #     # fix update for disconnected from the 'spp' group spp-guestappvm
           #     update_or_install_package('spp-guestappvm', True)
           #     update_or_install_package('spp-nmt', True)

            update_group(group)

        if options.mode =='checkyumserver': checkyumserver()
		
        # Added to remove postgresql RPMs
        if options.mode =='updatepostgresql': update_postgresql_removal()		

        if options.mode =='updagrademongodb': rolling_upgrade_For_MongoDB()		
        
        if options.mode =='updaterepourl': 
            if options.repourl is None:
                logger.warning ("The '--updaterepourl' option expects url , pass with '--repourl'" )
                failed_exit()
            else:
                update_repo_url(options.repourl)

    if options.mode == 'updatereposppv':

        if options.spp_version is None:
            logger.warning(
                "The '--updatereposppv' option expects version , pass with '--spp-version'")

            failed_exit()
        else:
            update_repo_env_spp_version(options.spp_version)
            yumcleanall()
    
    if options.mode == 'postreboot':
        logger.info("Started the Post Reboot Method")
        last_service_file = '/opt/SPP/update/execute_upgrade_steps.service'
        post_reboot_script = '/opt/SPP/update/post_reboot_execute_task.py'
        if options.rebootaction == 'verify':
            logger.info("Verifying and Transfering the file required to execute tasks after reboot")
            if os.path.exists(last_service_file):
                logger.info("Execute Upgrade Steps Service - execute_upgrade_steps.service exists at /opt/SPP/update/")
                copy_service_cmd = 'cp {} {}'.format(last_service_file, '/etc/systemd/system')
                os.popen(copy_service_cmd)
                logger.info("Execute Upgrade Steps Service - execute_upgrade_steps.service copied to /etc/systemd/system")
                os.popen("systemctl enable execute_upgrade_steps.service")
                logger.info("Enabled the execute_upgrade_steps Service")
            else:
                logger.info("Required file not found: /opt/SPP/update/execute_upgrade_steps.service")
            if os.path.exists(post_reboot_script):
                logger.info("Execute Upgrade Steps Script - post_reboot_execute_task.py exists at /opt/SPP/update/")
            else:
                logger.info("Required file not found: /opt/SPP/update/post_reboot_execute_task.py")
        if options.rebootaction == "execute":
            logger.info("Executing the Post Reboot Script")
            os.popen("python3 /opt/SPP/update/post_reboot_execute_task.py")
        if options.rebootaction == "remove":
            logger.info("Removing the Post reboot ")
            os.remove("/etc/systemd/system/execute_upgrade_steps.service")
        if options.rebootaction == "cleanup":
            logger.info("Cleaning Up the ISO")
            # Path to the ISO file
            iso_path = '/tmp/CUSTOM-RHEL9.5-x86_64.iso'
            cleanup_iso(iso_path)
            # Removing the SPP ISO stored on or after 17.X setup
            spp_iso_path = "/data/temp_uploads/spp-update.iso"
            cleanup_iso(spp_iso_path)
            # Removing the SPP ISO stored for setup earlier than 17.X
            os.popen("find /tmp/ecx* -name spp-update.iso -exec rm {} \; -print 2>/dev/null")

            # Find the complete rhel base ISO file inside `/data` and delete it if found
            complete_iso_file = os.popen("sudo find /data -maxdepth 1 -type f -name '*.iso' | head -n 1").read().strip()
            if complete_iso_file:
                cleanup_iso(complete_iso_file)

        if options.rebootaction == "executecreateifcfgfile":
            logger.info("Create ifcfg-ens160 file in upgraded RHEL9 SPP appliance")
            os.popen("sudo /opt/SPP/update/scripts/generate_ifcfg_ens160.sh")



def clean_exit():
    logger.info('Script completed successfully, Exiting!')
    sys.exit(0)

def cleanup_iso(iso_path):
    try:
        logger.info('Starting cleanup of ISO file...')

        if os.path.exists(iso_path):
            logger.info('Removing ISO file: %s', iso_path)
            os.remove(iso_path)  # Delete the ISO file
        else:
            logger.info('ISO file not found at: %s', iso_path)

        logger.info('ISO file cleanup completed.')

    except Exception as e:
        logger.error('Error during ISO cleanup: %s', e)

def rhel9Upgrade(path):
    logger.info('Initializing Leap Upgrade Process')

    # Define the path to the shell script
    shell_script = '/opt/SPP/update/scripts/leap_upgrade.sh'

    # Construct the full command with the path parameter passed to the script
    command = '{} "{}"'.format(shell_script, path)  # Using str.format() instead of f-strings

    # Run the shell script and capture output and error
    logger.info('Executing command: %s', command)
    p = Popen(command, stdout=PIPE, stderr=PIPE, shell=True)

    # Wait for the process to complete and get the return code
    stdout, stderr = p.communicate()  # This will wait for the process to finish

    # Log stdout and stderr for troubleshooting
    # logger.info("STDOUT: %s" % stdout.decode().strip())
    # logger.error("STDERR: %s" % stderr.decode().strip())

    # Check the return code to determine success or failure
    if p.returncode == 0:
        logger.info('Upgrade to RHEL 9 is done successfully.')
    else:
        logger.error('Upgrade to RHEL 9 failed.')
        logger.error("Process returned non-zero exit code: %s" % p.returncode)
    return p.returncode


def failed_exit():
    # logger.debug('Script completed with failures ......')
    sys.exit(1)

#
# updates /etc/yum/vars/spp_version file
#


def update_repo_env_spp_version(version):
    logger.info('Update environment variable for baseurl: ' + version)

    with open('/etc/yum/vars/spp_version', 'w') as file:
        file.write(version)


def update_repo_url(repourl):
    logger.info('Repo url to update ' + repourl)

    command = 'sed -i -e s#^baseurl=.*#baseurl=' + \
        repourl + '#g ' + repo.get_config_file()

    logger.info('Running command: ' + command)

    pobj = Popen(
        command,
        bufsize=1,
        shell=True,
        stdout=PIPE,
        stderr=PIPE,
        cwd='/tmp')
    out, err = pobj.communicate()
    out, err = decode_output(out, err)

    # logger.debug('Command Returncode : ' + str(pobj.returncode))

    if pobj.returncode == 0:
        if out:
            logger.info(out)
            clean_exit()
    else:
        if out:
            logger.info(out)

        if err:
            logger.error(err)

        failed_exit()


#
#  for internal use , does not exit
#    returns True if update is available, otherwise False
#
def is_update_available(group):
    packages = get_packages_in_group(group)

    if  packages:
            # Filter out packages related to MongoDB
        filtered_packages = [pkg for pkg in packages if 'mongo' not in pkg.lower()]
        logger.info('Filtered Packages_packages to skip mongo ')
        logger.info(filtered_packages)
        if not filtered_packages:
            logger.info('No packages found to check for updates after filtering MongoDB-related packages.')
            clean_exit()
        s = ' '.join(filtered_packages)
        logger.info("Filtered packages (excluding MongoDB)"+ s)

        command = YUM + ' -q --disablerepo=* --enablerepo=' + \
            repo.get_id() + ' check-update ' + s

        logger.info('Running command: ' + command)
        pobj = Popen(
            command,
            bufsize=1,
            shell=True,
            stdout=PIPE,
            stderr=PIPE,
            cwd='/tmp')
        out, err = pobj.communicate()
        out, err = decode_output(out, err)
        if pobj.returncode == 100:
            return True
        else:
            logger.info('No updates available for packages ' + s)

    return False


# is_ping(address)
#    arguments address
def is_ping(address):
    address = '8.8.8.8'
    retVal = False

    logger.debug("Ping address supplied: " + address)

    if address == '':
        return False

    command = PING + ' -c 1 ' + address + ' > /dev/null 2>&1'
    logger.debug("Ping check: " + command)
    rc = call([command], shell=True)
    if rc == 0:
        logger.debug('Ping Success')
        retVal = True
    else:
        logger.debug('Ping Failed')

    return retVal

# get_repo_server(repofile)
#       arguments full path to repo config


def get_repo_server(repofile):
    retVal = ''
    command = GREP + " baseurl " + repofile + \
        " | " + AWK + " -F'/' '{print $3}'"
    logger.debug('Running command: ' + command)

    return check_output([command], shell=True).strip()

#
# check_update()
#   arguments: group
#


def check_update(group):

    logger.info('Checking available update for group: ' + group)

    packages = get_packages_in_group(group)
    if not packages:
        logger.error('No packages in the group ' + group)
        failed_exit()

    # Filter out packages related to MongoDB
    filtered_packages = [pkg for pkg in packages if 'mongo' not in pkg.lower()]
    logger.info('Filtered packages for ')
    logger.info(filtered_packages)
    if not filtered_packages:
        logger.info('No packages found to check for updates after filtering MongoDB-related packages.')
        clean_exit()
    s = ' '.join(filtered_packages)
    logger.info("Filtered packages (excluding MongoDB)"+ s)

    command = YUM + ' -q --disablerepo=* --enablerepo=' + \
        repo.get_id() + '  check-update ' + s

    logger.info('Running command: ' + command)

    pobj = Popen(
        command,
        bufsize=1,
        shell=True,
        stdout=PIPE,
        stderr=PIPE,
        cwd='/tmp')
    out, err = pobj.communicate()
    out, err = decode_output(out, err)

    # logger.debug('Command Returncode : ' + str(pobj.returncode))

    if pobj.returncode == 100:   # update is available
        if out:
            logger.info(out)
            clean_exit()
    else:
        if out:
            logger.info(out)

        if err:
            logger.error(err)

        logger.error(
            'Attempt to upgrade to an older or current version of IBM Spectrum Protect Plus is not permitted.  Upgrade is terminating.')

        failed_exit()

#
# checks update from current repo or looking into 'nvr' tag file ( upgrade )
#  returns list with one or two elements
# Example:
# [{ "version": "10.1.4", build: "380"}, { "version": "10.1.6", "build": ""}]
#


def check_upgrade():

    update_version = {}
    upgrade_version = {}
    update_upgrade_list = []

    update_version = repo.get_spp_version()

    if update_version:  # if not empty and update is available
        update_upgrade_list.append(update_version)

    nsv = repo.get_next_supported_version()

    if nsv:
        nsv = nsv.decode()
        upgrade_version["version"] = nsv
        upgrade_version["build"] = ''
        
        update_upgrade_list.append(upgrade_version)

    update_upgrade_list_json = json.dumps(update_upgrade_list)

    # The following print statement gets converted into the Output in Admin-Console: checkECXUpdates method
    print (update_upgrade_list_json)


#
# add_new_packages_in_spp_group()
#
def add_new_packages_in_spp_group():

    update_or_install_package('spp-nmt', True)
    update_or_install_package('spp-k8s', True)
    update_or_install_package('spp-plugins-cm', True)
    update_or_install_package('spp-plugins-awsebs', True)
    update_or_install_package('spp-plugins-awsec2', True)
    # Replace openjdk with IBM java
    check_installed_openjdk()
    check_installed_ibmjava()
    # Check for old virgo installation
    check_installed_virgo()

    yum_groups_mark_packages(
        'spp',
        'spp-guestappvm spp-nmt spp-k8s spp-plugins-cm spp-plugins-awsebs spp-plugins-awsec2 vadp vadp-inst')

    # Check if update is available for next supported version - nsv
    if is_update_available('adminui'):
        update_group('adminui', True)

    yum_groups_mark_packages('adminui', 'spp-upst')


#
# update_group()
#
def update_group(group, continue_flag=False):

    if 'spp' in group:
        add_new_packages_in_spp_group()

    yum_group_info(group)

    command = YUM + ' --disablerepo=* --enablerepo=' + \
        repo.get_id() + ' -y groupupdate ' + group

    logger.info('Running command: ' + command)
    pobj = Popen(
        command,
        bufsize=1,
        shell=True,
        stdout=PIPE,
        stderr=PIPE,
        cwd='/tmp')

    stdout = pobj.stdout
    stderr = pobj.stderr
    while True:
        line = stdout.readline()
        line = line.decode('utf-8')
        logger.info(line)
        if line == '' and pobj.poll() is not None:
            break

    # logger.debug('Command Returncode : ' + str(pobj.returncode))

    if pobj.returncode == 0:
        if not continue_flag:
            clean_exit()
    else:

        while True:
            line = stderr.readline()
            line = line.decode('utf-8')
            logger.error(line)
            if line == '' and pobj.poll() is not None:
                break

        failed_exit()


#
#  check_installed_package()
#    returns true if package is installed
#
def check_installed_package(package):

    cmd = ("rpm -q " + package)
    p = Popen(cmd, stdout=PIPE, bufsize=1, shell=True, stderr=PIPE)
    stdout, stderr = p.communicate()
    stdout, stderr = decode_output(stdout, stderr)

    # print ( 'debug  return code: ' + str(p.returncode) )

    if p.returncode == 0:
        return True
    else:
        return False

#
#  install command
#

def install_package(package, continue_flag=False, install_type='yum'):
    LIBRARY_PATH = "/opt/ECX/library/"
    WHEELHOUSE_DIR = "/tmp/wheelhouse"
    GEM_TEMP_DIR = "/tmp/rubygems"
    
    def pip_install(source):
        try:
            files_to_install = []

            if os.path.isdir(source):
                for filename in os.listdir(source):
                    file_path = os.path.join(source, filename)
                    files_to_install.append(file_path)
            else:
                files_to_install.append(source)

            for file in files_to_install:
                cmd = [sys.executable, "-m", "pip", "install", "--no-deps", file]
                check_call(cmd)
                logger.info("Installed pip package:" + file)

        except Exception as e:
            logger.error("Failed to install pip packages from: " + str(source) + ". Error: " + str(e))

    if install_type == 'pip':
        package_path = os.path.join(LIBRARY_PATH, package + ".tar.gz")
        pip_install(package_path)

    elif install_type == 'wheelzip':
        zip_path = os.path.join(LIBRARY_PATH, "pydeps_bundle.zip")

        if not os.path.exists(zip_path):
            logger.error("Dependency zip file missing: " + zip_path)
            return

        try:
            if not os.path.exists(WHEELHOUSE_DIR):
                os.makedirs(WHEELHOUSE_DIR)

            logger.info("Unzipping dependencies from: " + zip_path)
            with zipfile.ZipFile(zip_path, 'r') as zip_ref:
                zip_ref.extractall(WHEELHOUSE_DIR)
            logger.info("Extraction completed")

            pip_install(WHEELHOUSE_DIR)
        except Exception as e:
            logger.error("Failed to install packages from pydeps_bundle.zip: " + str(e))

    elif install_type == 'rubygemzip':
        zip_path = os.path.join(LIBRARY_PATH, "ruby_gems.zip")

        if not os.path.exists(zip_path):
            logger.error("Dependency zip file missing: " + zip_path)
            return

        try:
            if not os.path.exists(GEM_TEMP_DIR):
                os.makedirs(GEM_TEMP_DIR)

            logger.info("Unzipping Ruby gems from: " + zip_path)
            with zipfile.ZipFile(zip_path, 'r') as zip_ref:
                zip_ref.extractall(GEM_TEMP_DIR)
            logger.info("Ruby gems zip extraction completed")

            check_call("gem install --local *.gem", cwd=GEM_TEMP_DIR, shell=True)
            logger.info("Successfully installed all Ruby gems with dependency resolution")

        except Exception as e:
            logger.error("Failed to install Ruby gems from ruby_gems.zip: " + str(e))


    else:
        command = YUM + ' --disablerepo=* --enablerepo=' + \
            repo.get_id() + ' -y install ' + package
        execute_yum_command(command, continue_flag)


#
# update_package()
#   update all if package is empty
#
def update_or_install_package(
        package, continue_flag=False, skip_gpg_check=False):

    YUM_OPTION = ''

    if skip_gpg_check:
        YUM_OPTION += ' --nogpgcheck '

    if package == '':
        # only updates are allowed

        logger.info(
            'Updating all packages from ' +
            repo.get_id() +
            ' repository')
        command = YUM + YUM_OPTION + ' --disablerepo=* --enablerepo=' + repo.get_id() + \
            ' -y update '

    else:

        # package is not found, install the package
        if check_installed_package(package):
            logger.info('Updating package:  ' + package)
            command = YUM + YUM_OPTION + ' --disablerepo=* --enablerepo=' + \
                repo.get_id() + ' -y update ' + package

        else:
            logger.info('Installing package:  ' + package)
            command = YUM + YUM_OPTION + ' --disablerepo=* --enablerepo=' + \
                repo.get_id() + ' -y install ' + package
    execute_yum_command(command, continue_flag)

    if package == 'rabbitmq-server':
        # Replace /var/run to /run in rabbitmq-server.conf
        replace_command = "sed -i 's|/var/run|/run|g' /usr/lib/tmpfiles.d/rabbitmq-server.conf"
        os.popen(replace_command)


def execute_yum_command(command, continue_flag=False):

    logger.info('Running command: ' + command)
    pobj = Popen(
        command,
        bufsize=1,
        shell=True,
        stdout=PIPE,
        stderr=PIPE,
        cwd='/tmp')

    stdout = pobj.stdout
    stderr = pobj.stderr

    while True:
        line = stdout.readline()
        line = line.decode('utf-8')
        logger.info(line)
        if line == '' and pobj.poll() is not None:
            break

    # logger.debug('Command Returncode : ' + str(pobj.returncode))

    if pobj.returncode == 0:
        if not continue_flag:
            clean_exit()
    else:

        while True:
            line = stderr.readline()
            line = line.decode('utf-8')
            logger.error(line)
            if line == '' and pobj.poll() is not None:
                break

        failed_exit()

#
# run_vsnap_dist_installer()
#    runs /opt/vsnap-dist/vsnap-dist-*.run
#


def run_vsnap_dist_installer():

    list = glob.glob('/opt/vsnap-dist/vsnap-dist-*.run')

    if list:  # installer exists
        installer = list[0]
        command = installer + '  -- nostart'
        pobj = Popen(
            command,
            bufsize=1,
            shell=True,
            stdout=PIPE,
            stderr=PIPE,
            cwd='/tmp')

        stdout = pobj.stdout
        stderr = pobj.stderr
        logger.info('Running command: ' + command)

        while True:
            line = stdout.readline()
            line = line.decode('utf-8')
            logger.info(line)
            if line == '' and pobj.poll() is not None:
                break

        # logger.debug('Command Returncode : ' + str(pobj.returncode))

        if pobj.returncode == 0:
            logger.info('Successful installation of vsnap distribution')
        else:

            while True:
                line = stderr.readline()
                line = line.decode('utf-8')
                logger.error(line)
                if line == '' and pobj.poll() is not None:
                    break


#
# get_packages_in_group()
#
def get_packages_in_group(group):

    packages = []

    cmd = YUM + ' -q --disablerepo=* --enablerepo=' + \
        repo.get_id() + ' groupinfo ' + group

    logger.info('Running command: ' + cmd)
    start_read = 0
    p = Popen(cmd, stdout=PIPE, bufsize=1, shell=True, stderr=PIPE)
    stdout = p.stdout
    stderr = p.stderr
    with stdout:
        for line in iter(stdout.readline, b''):
            line = line.decode('utf-8')
            if line.startswith(" Mandatory Packages:"):
                start_read = 1
                continue
            if line.startswith(" Installed Packages:"):
                continue

            if start_read:
                packages.append(line.strip().strip('=').strip('-').strip('+'))
    p.wait()  # wait for the subprocess to exit

    return packages


def checkyumserver():
    status = repo.get_http_server_code()
    if str(status) == '200':
        return 0
    else:
        failed_exit()


def yumcleanall():
    logger.info('Cleaning yum cache...with expire-cache')
    cmd = [YUM, '--enablerepo=*', 'clean', 'all', 'expire-cache']
    Popen(cmd).wait()


def yum_convert_groups_to_objects():
    logger.info('Converting yum packages to objects.')
    cmd = [
        YUM,
        '--disablerepo=*',
        '--enablerepo=' +
        repo.get_id(),
        'groups',
        '--setopt=group_command=objects',
        'mark-convert']
    Popen(cmd).wait()


def yum_group_info(group):
    logger.info('Group info for ' + group)
    cmd = [
        YUM,
        '--disablerepo=*',
        '--enablerepo=' +
        repo.get_id(),
        'groupinfo',
        group]
    Popen(cmd).wait()


def check_installed_openjdk():
    logger.info('Checking for previously installed openjdk')
    cmd = 'rpm -q java-1.8.0-openjdk java-1.8.0-openjdk-devel java-1.8.0-openjdk-headless'
    logger.info('Running command: ' + cmd)
    p = Popen(cmd, stdout=PIPE, bufsize=1, shell=True, stderr=PIPE)
    stdout, stderr = p.communicate()
    stdout, stderr = decode_output(stdout, stderr)
    logger.info('debug  return code: ' + str(p.returncode))
    if p.returncode == 0:
        rcmd = [
            'rpm',
            '-e',
            'java-1.8.0-openjdk',
            'java-1.8.0-openjdk-devel',
            'java-1.8.0-openjdk-headless']
        Popen(rcmd).wait()
    else:
        logger.info('No installed openjdk detected.')


def check_installed_ibmjava():
    logger.info('Checking for previously installed IBM java')
    cmd = 'rpm -q ibm-java-x86_64-sdk'
    logger.info('Running command: ' + cmd)
    p = Popen(cmd, stdout=PIPE, bufsize=1, shell=True, stderr=PIPE)
    stdout, stderr = p.communicate()
    stdout, stderr = decode_output(stdout, stderr)
    logger.info('debug  return code: ' + str(p.returncode))
    if p.returncode == 0:
        logger.info('IBM Java detected.')
    else:
        logger.info('Installing IBM Java.')
        update_or_install_package('ibm-java-x86_64-sdk', True)
        updatejavaalt()


def updatejavaalt():
    logger.info('Updating java alternatives file...')
    cmd = [UPDALT, '--install', '/usr/bin/java', 'java', '/opt/ibm/java-x86_64-80/bin/java', '1',
           '--slave', '/usr/bin/jjs', 'jjs', '/opt/ibm/java-x86_64-80/bin/jjs',
           '--slave', '/usr/bin/keytool', 'keytool', '/opt/ibm/java-x86_64-80/bin/keytool',
           '--slave', '/usr/bin/pack200', 'pack200', '/opt/ibm/java-x86_64-80/bin/pack200',
           '--slave', '/usr/bin/policytool', 'policytool', '/opt/ibm/java-x86_64-80/bin/policytool',
           '--slave', '/usr/bin/rmid', 'rmid', '/opt/ibm/java-x86_64-80/bin/rmid',
           '--slave', '/usr/bin/rmiregistry', 'rmiregistry', '/opt/ibm/java-x86_64-80/bin/rmiregistry',
           '--slave', '/usr/bin/tnameserv', 'tnameserv', '/opt/ibm/java-x86_64-80/bin/tnameserv',
           '--slave', '/usr/bin/unpack200', 'unpack200', '/opt/ibm/java-x86_64-80/bin/unpack200']
    Popen(cmd).wait()
    logger.info('Setting java alternatives file...')
    cmd = [UPDALT, '--set', 'java', '/opt/ibm/java-x86_64-80/bin/java']
    Popen(cmd).wait()
    logger.info('Updating javac alternatives file...')
    cmd = [UPDALT, '--install', '/usr/bin/javac', 'javac', '/opt/ibm/java-x86_64-80/bin/javac', '1',
           '--slave', '/usr/bin/appletviewer', 'appletviewer', '/opt/ibm/java-x86_64-80/bin/appletviewer',
           '--slave', '/usr/bin/ControlPanel', 'ControlPanel', '/opt/ibm/java-x86_64-80/bin/ControlPanel',
           '--slave', '/usr/bin/extcheck', 'extcheck', '/opt/ibm/java-x86_64-80/bin/extcheck',
           '--slave', '/usr/bin/idlj', 'idlj', '/opt/ibm/java-x86_64-80/bin/idlj',
           '--slave', '/usr/bin/jar', 'jar', '/opt/ibm/java-x86_64-80/bin/jar',
           '--slave', '/usr/bin/jarsigner', 'jarsigner', '/opt/ibm/java-x86_64-80/bin/jarsigner',
           '--slave', '/usr/bin/javadoc', 'javadoc', '/opt/ibm/java-x86_64-80/bin/javadoc',
           '--slave', '/usr/bin/javah', 'javah', '/opt/ibm/java-x86_64-80/bin/javah',
           '--slave', '/usr/bin/javap', 'javap', '/opt/ibm/java-x86_64-80/bin/javap',
           '--slave', '/usr/bin/javaw', 'javaw', '/opt/ibm/java-x86_64-80/bin/javaw',
           '--slave', '/usr/bin/javaws', 'javaws', '/opt/ibm/java-x86_64-80/bin/javaws',
           '--slave', '/usr/bin/jconsole', 'jconsole', '/opt/ibm/java-x86_64-80/bin/jconsole',
           '--slave', '/usr/bin/jcontrol', 'jcontrol', '/opt/ibm/java-x86_64-80/bin/jcontrol',
           '--slave', '/usr/bin/jdb', 'jdb', '/opt/ibm/java-x86_64-80/bin/jdb',
           '--slave', '/usr/bin/jdeps', 'jdeps', '/opt/ibm/java-x86_64-80/bin/jdeps',
           '--slave', '/usr/bin/jdmpview', 'jdmpview', '/opt/ibm/java-x86_64-80/bin/jdmpview',
           '--slave', '/usr/bin/jrunscript', 'jrunscript', '/opt/ibm/java-x86_64-80/bin/jrunscript',
           '--slave', '/usr/bin/native2ascii', 'native2ascii', '/opt/ibm/java-x86_64-80/bin/native2ascii',
           '--slave', '/usr/bin/rmic', 'rmic', '/opt/ibm/java-x86_64-80/bin/rmic',
           '--slave', '/usr/bin/schemagen', 'schemagen', '/opt/ibm/java-x86_64-80/bin/schemagen',
           '--slave', '/usr/bin/serialver', 'serialver', '/opt/ibm/java-x86_64-80/bin/serialver',
           '--slave', '/usr/bin/wsgen', 'wsgen', '/opt/ibm/java-x86_64-80/bin/wsgen',
           '--slave', '/usr/bin/wsimport', 'wsimport', '/opt/ibm/java-x86_64-80/bin/wsimport',
           '--slave', '/usr/bin/xjc', 'xjc', '/opt/ibm/java-x86_64-80/bin/xjc',
           '--slave', '/usr/bin/ikeycmd', 'ikeycmd', '/opt/ibm/java-x86_64-80/jre/bin/ikeycmd',
           '--slave', '/usr/bin/ikeyman', 'ikeyman', '/opt/ibm/java-x86_64-80/jre/bin/ikeyman',
           '--slave', '/usr/bin/j9vm', 'j9vm', '/opt/ibm/java-x86_64-80/jre/bin/j9vm',
           '--slave', '/usr/bin/java-rmi.cgi', 'java-rmi.cgi', '/opt/ibm/java-x86_64-80/jre/bin/java-rmi.cgi',
           '--slave', '/usr/bin/jextract', 'jextract', '/opt/ibm/java-x86_64-80/jre/bin/jextract',
           '--slave', '/usr/bin/kinit', 'kinit', '/opt/ibm/java-x86_64-80/jre/bin/kinit',
           '--slave', '/usr/bin/klist', 'klist', '/opt/ibm/java-x86_64-80/jre/bin/klist',
           '--slave', '/usr/bin/ktab', 'ktab', '/opt/ibm/java-x86_64-80/jre/bin/ktab']
    Popen(cmd).wait()
    logger.info('Setting javac alternatives file...')
    cmd = [UPDALT, '--set', 'javac', '/opt/ibm/java-x86_64-80/bin/javac']
    Popen(cmd).wait()
    mkdcmd = ['mkdir', '-p', '/usr/lib/jvm/jre/bin']
    Popen(mkdcmd).wait()
    lnkcmd = [
        'ln',
        '-s',
        '/opt/ibm/java-x86_64-80/jre/bin/java',
        '/usr/lib/jvm/jre/bin/java']
    Popen(lnkcmd).wait()
    mkdjcmd = ['mkdir', '-p', '/usr/java']
    Popen(mkdjcmd).wait()
    lnkjcmd = ['ln', '-s', '/opt/ibm/java-x86_64-80', '/usr/java/latest']
    Popen(lnkjcmd).wait()


def check_installed_virgo():
    logger.info(
        'Checking for previously installed virgo-tomcat virgo-plugins rpms')
    cmd = 'rpm -q virgo-tomcat virgo-plugins'
    logger.info('Running command: ' + cmd)
    p = Popen(cmd, stdout=PIPE, bufsize=1, shell=True, stderr=PIPE)
    stdout, stderr = p.communicate()
    stdout, stderr = decode_output(stdout, stderr)
    logger.info('debug return code: ' + str(p.returncode))
    if p.returncode == 0:
        logger.info('Backup virgo files')
        stopvircmd = ['systemctl', 'stop', 'virgo']
        Popen(stopvircmd).wait()
        mkdtmpcmd = ['mkdir', '-p', '/tmp/sppvirgo/']
        Popen(mkdtmpcmd).wait()
        cpkeycmd = [
            'cp',
            '-fp',
            '/opt/virgo/configuration/keystore',
            '/tmp/sppvirgo/']
        Popen(cpkeycmd).wait()
        cpecrtcmd = [
            'cp',
            '-fp',
            '/opt/virgo/configuration/ecx-beta.crt',
            '/tmp/sppvirgo/']
        Popen(cpecrtcmd).wait()
        cpekeycmd = [
            'cp',
            '-fp',
            '/opt/virgo/configuration/ecx-beta.key',
            '/tmp/sppvirgo/']
        Popen(cpekeycmd).wait()
        cplogcmd = [
            'cp',
            '-fpr',
            '/opt/virgo/serviceability/logs',
            '/tmp/sppvirgo/']
        Popen(cplogcmd).wait()
        cpliccmd = [
            'cp',
            '-fpr',
            '/opt/virgo/repository/ecx-usr/ECX.lic',
            '/tmp/sppvirgo/']
        Popen(cpliccmd).wait()
        chcrtcmd = ['chown', 'virgo:sppcerts', '/tmp/sppvirgo/ecx-beta.crt']
        Popen(chcrtcmd).wait()
        chkeycmd = ['chown', 'virgo:sppcerts', '/tmp/sppvirgo/ecx-beta.key']
        Popen(chkeycmd).wait()
        yumrmcmd = YUM + ' remove --disablerepo=* --enablerepo=' + \
            repo.get_id() + ' -y virgo-tomcat virgo-plugins tcnative2 spp'
        logger.info('Running command: ' + yumrmcmd)
        p = Popen(yumrmcmd, stdout=PIPE, bufsize=1, shell=True, stderr=PIPE)
        stdout, stderr = p.communicate()
        stdout, stderr = decode_output(stdout, stderr)
        logger.info('Installing spp-virgo')
        update_or_install_package('spp-virgo', True)
        restore_virgo_files()
    else:
        logger.info('No installed virgo-tomcat virgo-plugins detected.')


def restore_virgo_files():
    logger.info('Restore virgo files from backup')
    stopvircmd = ['systemctl', 'stop', 'virgo']
    Popen(stopvircmd).wait()
    mvkeycmd = ['mv', '/tmp/sppvirgo/keystore', '/opt/virgo/configuration/']
    Popen(mvkeycmd).wait()
    mvecrtcmd = [
        'mv',
        '/tmp/sppvirgo/ecx-beta.crt',
        '/opt/virgo/configuration/']
    Popen(mvecrtcmd).wait()
    mvekeycmd = [
        'mv',
        '/tmp/sppvirgo/ecx-beta.key',
        '/opt/virgo/configuration/']
    Popen(mvekeycmd).wait()
    mvliccmd = [
        'mv',
        '/tmp/sppvirgo/ECX.lic',
        '/opt/ECX/virgo/repository/usr/']
    Popen(mvliccmd).wait()
    cplogcmd = [
        'cp',
        '-fpr',
        '/tmp/sppvirgo/logs/*',
        '/opt/virgo/serviceability/logs/']
    Popen(cplogcmd).wait()
    stvirgocmd = ['systemctl', 'start', 'virgo']
    Popen(stvirgocmd).wait()


def update_postgresql_removal():
    logger.info(
        'Checking for previously installed postgresql14-server, postgresql14, postgresql14-libs RPMs')
    cmd = 'rpm -q postgresql14-server postgresql14-14.3 postgresql14-libs'
    logger.info('Running command: ' + cmd)
    p = Popen(cmd, stdout=PIPE, bufsize=1, shell=True, stderr=PIPE)
    stdout, stderr = p.communicate()
    stdout, stderr = decode_output(stdout, stderr)
    logger.info('debug return code: ' + str(p.returncode))
    if p.returncode == 0:
        logger.info('Run migration script psql2mongo.sh')
        shell_script = '/opt/SPP/update/scripts/psql2mongo.sh'
        p = Popen(
            shell_script,
            stdout=PIPE,
            bufsize=1,
            shell=True,
            stderr=PIPE)
        stdout, stderr = p.communicate()
        logger.info('debug return code: ' + str(p.returncode))
        if p.returncode == 0:
            logger.info('Migration script ran successfully.')
            yumrmcmd = YUM + ' remove --disablerepo=* --enablerepo=' + \
                repo.get_id() + ' -y postgresql14-server postgresql14-14.3 postgresql14-libs'
            logger.info('Running command: ' + yumrmcmd)
            p = Popen(
                yumrmcmd,
                stdout=PIPE,
                bufsize=1,
                shell=True,
                stderr=PIPE)
            stdout, stderr = p.communicate()
            stdout, stderr = decode_output(stdout, stderr)
            logger.info('debug return code: ' + str(p.returncode))
            if p.returncode == 0:
                logger.info('PostgreSQL packages removed successfully.')
            else:
                logger.info('stderr: ' + stderr)
        else:
            logger.error(
                'Migration script failed with return code: ' + str(p.returncode))
    else:
        logger.info(
            'No PostgreSQL packages detected - postgresql14-server, postgresql14, postgresql14-libs RPMs.')

def rolling_upgrade_For_MongoDB():
        logger.info('Rolling Mongodb upgrade Initialization')
        shell_script = '/opt/SPP/update/scripts/mongo_upgrade.sh'
        p = Popen(shell_script, stdout=PIPE, bufsize=1, shell=True, stderr=PIPE)
        if p.returncode == 0:
            logger.info('Upgrade Mongo binaries done successfully.')
        else:
            logger.error('Upgrade Mongo binaries failed or system has older Mongo version.')


def yum_groups_mark_packages(group,packages):
    logger.info('yum_groups_mark_packages  for ' + group)
    package_list = packages.split()
    cmd = [DNF, 'mark', 'group'] + package_list
    logger.info(cmd)
    Popen(cmd).wait()

def check_build_to_build_upgrade(mode):
    if mode == 'updateos':
        if repo.get_id() == "osupdate":
            repo_name = "localrepo"
        else:
            repo_name = "spprepo"
    else:
        repo_name = repo.get_id()

    try :
        version_dict = repo.get_spp_version(repo_name)
        upgrade_version = version_dict['version']
        #upgrade_build = version_dict['build']
        current_version=os.popen("rpm -q spp-release | awk -F'-' '{print $3}' | tr '\n' ' '").read()[:-1]
        #current_build=os.popen("rpm -q spp-release | awk -F'-' '{print $4}' | awk -F'.' '{print $1}'")

        if upgrade_version == current_version:
            return True
    except Exception as e:
        logger.error("Cannot fetch the SPP Versions: " + str(e))
    return False


#
# Represents yum repository
#
class Repo:
    'Class for yumrepository'

    def __init__(self, repo='localrepo'):

        self.group_name = 'spp'

        self.id = repo
        cmd = YUM + ' repoinfo ' + repo

        p = Popen(cmd, stdout=PIPE, bufsize=1, shell=True, stderr=PIPE)
        stdout, stderr = p.communicate()
        stdout, stderr = decode_output(stdout, stderr)

        for line in stdout.splitlines():
            if line.startswith("Repo-name"):
                self.name = line.split(':')[1]
            if line.startswith("Repo-status"):
                self.status = line.split(':')[1]
            if line.startswith("Repo-baseurl"):
                self.baseurl = line.split(' : ')[1]
                if not self.baseurl.endswith('/'):
                    self.baseurl += '/'

        # print (self.baseurl )

    def get_id(self):
        return self.id

    def get_baseurl(self):
        return self.baseurl

    def get_group_name(self):
        return self.group_name

    def get_config_file(self):
        return self.config

    def get_http_server_code(self):

        if self.baseurl.startswith("http"):
            code = urllib.request.urlopen(self.baseurl).getcode()

            logger.debug(
                'HTTP status code (' +
                self.baseurl +
                ')  : ' +
                str(code))

        else:
            # check internet connection anyway
            code = urllib.request.urlopen('http://www.google.com').getcode()
            logger.info(
                'Local repository ' +
                self.id +
                ' wad detected , checking internet connectivity anyway: ' +
                str(code))

        return code

    #
    # reads 'nsv' file, supplied with image or updated manually
    #
    def get_next_supported_version(self):

        version = ''
        target_url = self.baseurl + 'nsv'

        try:
            f = urllib.request.urlopen(target_url)
            read_data = f.read()
            version = read_data.strip()
        except BaseException:
            logger.warning(
                'Failed to open url ' +
                target_url +
                ' , or file does not exist')

        return version

    #
    # resolve version from release package
    # yum check-update returns:
    #  spp-release.noarch  <version>  <reponame>
    #
    # retrns dictionary:
    #     {'version': '999.0.0', 'build': '1228'}

    def get_spp_version(self, repo_name=None):

        release_package = 'spp-release'

        dict = {}

        if repo_name is None:
            repo_name = repo.get_id()

        try:

            cmd = YUM + ' -q --disablerepo=* --enablerepo=' + \
                repo_name + ' check-update  ' + release_package

            if options.verbosity > 1:
                logger.info('Running command: ' + cmd)

            p = Popen(cmd, stdout=PIPE, bufsize=1, shell=True, stderr=PIPE)
            stdout, stderr = p.communicate()
            stdout, stderr = decode_output(stdout, stderr)

            m = re.match(
                r'.*' +
                release_package +
                '.\\S+\\s+(\\S+).*',
                stdout,
                re.S)
            if (m):

                # logger.info('Version available on update: ' + m.group(1) )
                version, build = m.group(1).split('-')
                dict['version'] = version
                dict['build'] = build

            else:
                logger.warning(
                    'Can not detect version eligible for update :  ' + cmd)

        except Exception as e:
            logger.warning(e)

        return dict


if __name__ == "__main__":

    version = "1.8.3"
    parser = OptionParser()
    usage = "usage: %prog [options]"
    parser = OptionParser(usage=usage, version="%prog " + version)

    parser.add_option("-l", "--log",
                      default='/data/log/adminconsole/update-manager.log',
                      dest='log',
                      help="Log file location"
                      " [ default: %default ]")

    parser.add_option("--group",
                      dest='group',
                      help=SUPPRESS_HELP)

    parser.add_option("--package",
                      dest='package',
                      help=SUPPRESS_HELP)

    parser.add_option(
        "-v",
        action="count",
        dest="verbosity",
        default=0,
        help=SUPPRESS_HELP)

    # undocumented option:  updatereposppv [update yum repo env vars for
    # spp_version]
    parser.add_option("--mode",
                      help="supported mode: checkupdate, checkupgrade, update, updateos, checkyumserver, updaterepourl, install, postreboot")

    parser.add_option(
        "--yumclean",
        action="store_true",
        default=False,
        dest='yumclean',
        help=SUPPRESS_HELP)

    parser.add_option("--repourl",
                      dest='repourl',
                      help=SUPPRESS_HELP)

    parser.add_option("--skip-gpg-check",
                      action="store_true",
                      dest='skip_gpg_check',
                      default=False,
                      help=SUPPRESS_HELP)

    parser.add_option("--spp-version",
                      dest='spp_version',
                      help=SUPPRESS_HELP)


    parser.add_option('--upgrade', dest='upgrade', help='Specify the upgrade type (e.g., rhel9)')

    parser.add_option('--rebootaction', dest='rebootaction', help='Specify the action to be performed. During the Post Upgrade. Supported Mode: cleanup, remove, verify')

    parser.add_option('--install-type', dest='install_type', default='yum', help='Package installer. Supported Mode: yum, pip. Default: yum')

    parser.add_option("--skip-step",action="store_true", dest="skip_step", default=False, help="Skip the step in case of Build to Build upgrade Default: False")

    (options, args) = parser.parse_args()

    if options.mode is None:
        print("A mandatory option 'mode' is missing\n")
        parser.print_help()
        failed_exit()

    setup()

    if options.verbosity > 1:
        logger.info(
            '+----------------------- STARTING UPDATE MANAGER --------------------------------+')
        logger.info(os.path.realpath(__file__) + ' called with options:')
        logger.info(options)

    if options.mode == 'updateos' and os.path.exists(
            '/etc/yum.repos.d/local.repo'):
        repo = Repo('osupdate')

    elif os.path.exists('/etc/yum.repos.d/local.repo'):
        repo = Repo('localrepo')

    elif options.mode == 'updateos' and os.path.exists(OSUPDATE_REPO_CONFIG) and is_ping(get_repo_server(OSUPDATE_REPO_CONFIG)):
        repo = Repo('osupdaterepo')

    elif os.path.exists(SPP_REPO_CONFIG) and is_ping(get_repo_server(SPP_REPO_CONFIG)):
        repo = Repo('spprepo')
    else:
        print('Can not detect yum repository')
        failed_exit()

    if options.skip_step and check_build_to_build_upgrade(options.mode):
        print("Skipping the step for Build to Build upgrade")
    else:
        main()
